@@ -5,9 +5,6 @@
 
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
 
-#ifndef CGAL_NEWER_API_4_11
-#include <CGAL/Regular_triangulation_euclidean_traits_3.h>
-#endif
 #include <CGAL/Regular_triangulation_3.h>
 #include <CGAL/Alpha_shape_3.h>
 #include <CGAL/Triangulation_vertex_base_with_info_3.h>
@@ -22,7 +19,6 @@ struct VertexInfo {
 };
 
 typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
-#ifdef CGAL_NEWER_API_4_11
 typedef CGAL::Regular_triangulation_vertex_base_3<Kernel>        Rvb;
 typedef CGAL::Triangulation_vertex_base_with_info_3<VertexInfo, Kernel, Rvb> Vinfo;
 typedef CGAL::Alpha_shape_vertex_base_3<Kernel,Vinfo>              Vb;
@@ -31,15 +27,6 @@ typedef CGAL::Alpha_shape_cell_base_3<Kernel,Rcb>                Cb;
 typedef CGAL::Triangulation_data_structure_3<Vb,Cb>         Tds;
 typedef CGAL::Regular_triangulation_3<Kernel,Tds>                Triangulation;
 typedef CGAL::Alpha_shape_3<Triangulation>                CGAL_AlphaShape3;
-#else
-typedef CGAL::Regular_triangulation_euclidean_traits_3<Kernel> Gt;
-typedef CGAL::Triangulation_vertex_base_with_info_3<VertexInfo, Gt> Vinfo;
-typedef CGAL::Alpha_shape_vertex_base_3<Gt, Vinfo> Vb;
-typedef CGAL::Alpha_shape_cell_base_3<Gt> Fb;
-typedef CGAL::Triangulation_data_structure_3<Vb, Fb> Tds;
-typedef CGAL::Regular_triangulation_3<Gt, Tds> Triangulation;
-typedef CGAL::Alpha_shape_3<Triangulation> CGAL_AlphaShape3;
-#endif
 typedef std::pair<CGAL_AlphaShape3::Weighted_point, VertexInfo> PointWithInfo;
 
 
@@ -893,7 +880,6 @@ static bool Cell_equal(PyObject* o1, PyObject* o2) {
   return c1->alpha_ == c2->alpha_ && c1->handle_ == c2->handle_;
 }
 
-#if PY_MAJOR_VERSION >= 3
 static PyModuleDef alpha_shape3_Module = {
   PyModuleDef_HEAD_INIT,
   "homcloud.alpha_shape3",
@@ -902,37 +888,24 @@ static PyModuleDef alpha_shape3_Module = {
   NULL, NULL, NULL, NULL, NULL
 };
 
-#define INIT_ERROR return NULL
-#else
-#define INIT_ERROR return
-#endif
-
-
-#if PY_MAJOR_VERSION >= 3
 PyMODINIT_FUNC
 PyInit_alpha_shape3()
-#else
-extern "C" void initalpha_shape3()
-#endif
 {
   if (PyType_Ready(&AlphaShape3Type) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&VertexType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&CellType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&EdgeType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&FacetType) < 0)
-    INIT_ERROR;
+    return NULL;
 
-#if PY_MAJOR_VERSION >= 3
   PyObject* module = PyModule_Create(&alpha_shape3_Module);
-#else
-  PyObject* module = Py_InitModule("homcloud.alpha_shape3", NULL);
-#endif
+
   if (!module)
-    INIT_ERROR;
+    return NULL;
 
   Py_INCREF(&AlphaShape3Type);
   PyModule_AddObject(module, "AlphaShape3", cast_PyObj(&AlphaShape3Type));
@@ -947,9 +920,7 @@ extern "C" void initalpha_shape3()
   
   import_array();
 
-#if PY_MAJOR_VERSION >= 3
   return module;
-#endif
 }
 
 
