@@ -1,4 +1,3 @@
-#include "homcloud_common.h"
 #include <cstdint>
 #include <cmath>
 #include <vector>
@@ -9,6 +8,9 @@
 #include <sstream>
 #include <algorithm>
 
+#include "homcloud_common.h"
+#include "homcloud_numpy.h"
+
 #define HOMCLOUD_PHAT_EXT_MODULE
 #include "phat_ext.h"
 
@@ -27,19 +29,22 @@ struct StringWriter {
   }
 };
 
-struct CubicalFiltrationExt {
-  PyObject_HEAD
-  PyArrayObject* array_;
-  PyObject* periodic_;
-  int ndim_;
-  npy_intp* shape_;
-  std::vector<npy_intp>* required_bits_;
-  std::vector<npy_intp>* bottom_bit_;
-  std::vector<double>* cache_value_at_;
-  std::vector<CubeID>* sorted_cubes_;
-  CubeID cubedim_mask_;
-  bool indexized_;
-  bool save_boundary_map_;
+class CubeEncoder {
+ public:
+  CubeEncoder(int ndim, const npy_intp* shape):
+      ndim_(ndim), shape_(shape, shape + ndim),
+      required_bits_(ndim), bottom_bit_(ndim + 1)
+  {
+    bottom_bit_[0] = 0;
+    
+    for (int n = 0; n < ndim_; ++n) {
+      npy_intp bits = RequiredBits1D(shape_[n]);
+      required_bits_[n] = bits;
+      bottom_bit_[n + 1] = bottom_bit_[n] + bits;
+    }
+
+    cubedim_mask_ = BuildCubeDimMask();
+  }
 
   static npy_intp RequiredBits1D(npy_intp n) {
     for (npy_intp k = 0; k < 32; ++k)
@@ -51,92 +56,57 @@ struct CubicalFiltrationExt {
   CubeID BuildCubeDimMask() {
     CubeID mask = 0;
     for (int k = 0; k < ndim_; ++k)
-      mask |= 1 << (bottom_bit_->at(k));
+      mask |= 1 << bottom_bit_[k];
     return mask;
   }
 
-  void SetUp(PyArrayObject* array, PyObject* periodic, bool save_boundary_map) {
-    ndim_ = PyArray_NDIM(array);
-    shape_ = PyArray_DIMS(array);
-    array_ = array;
-    periodic_ = periodic;
-    Py_INCREF(array_);
-    Py_INCREF(periodic_);
-
-    required_bits_ = new std::vector<npy_intp>(ndim_);
-    bottom_bit_ = new std::vector<npy_intp>(ndim_ + 1);
-
-    bottom_bit_->at(0) = 0;
-    for (int n = 0; n < ndim_; ++n) {
-      npy_intp bits = RequiredBits1D(shape_[n]);
-      required_bits_->at(n) = bits;
-      bottom_bit_->at(n + 1) = bottom_bit_->at(n) + bits;
-    }
-
-    cache_value_at_ = new std::vector<double>(1 << bottom_bit_->at(ndim_),
-                                             std::numeric_limits<double>::quiet_NaN());
-
-    cubedim_mask_ = BuildCubeDimMask();
-    indexized_ = false;
-    save_boundary_map_ = save_boundary_map;
-  }
-
-  void TearDown() {
-    Py_XDECREF(array_);
-    Py_XDECREF(periodic_);
-    delete required_bits_; required_bits_ = nullptr;
-    delete bottom_bit_; bottom_bit_ = nullptr;
-    delete cache_value_at_; cache_value_at_ = nullptr;
-    delete sorted_cubes_; sorted_cubes_ = nullptr;
-  }
-
-  CubeID Mask(int n) {
-    return ((1 << required_bits_->at(n)) - 1);
-  }
-
-  CubeID Encode(const std::vector<npy_intp>& coords,
-                const std::vector<npy_intp>& non_degenerate) const {
-    CubeID id = 0;
-    for (int n = ndim_ - 1; n >= 0; --n) {
-      id = (id << required_bits_->at(n)) | (coords[n] << 1) | non_degenerate[n];
-    }
-    return id;
-  }
-
-  void Decode(CubeID id,
-              std::vector<npy_intp>* coords,
-              std::vector<npy_intp>* non_degenerate) {
-    for (int n = 0; n < ndim_; ++n) {
-      CubeID slice = id & Mask(n);
-      coords->at(n) = slice >> 1;
-      non_degenerate->at(n) = slice & 1;
-      id >>= required_bits_->at(n);
-    }
-  }
-
-  int CubeDim(CubeID id) {
+  int CubeDim(CubeID id) const {
 #ifdef __GNUC__
     return __builtin_popcountll(id & cubedim_mask_);
 #else
     int dim = 0;
     for (int n = 0; n < ndim_; ++n) {
       dim += (id & Mask(n)) & 1;
-      id >>= required_bits_->at(n);
+      id >>= required_bits_[n];
     }
     return dim;
 #endif
   }
 
-  CubeID BuildFacet(CubeID partial, CubeID coface, CubeID mask, int n) {
-    return (coface & ~mask) | (partial << (bottom_bit_->at(n) + 1));
+  CubeID Encode(const std::vector<npy_intp>& coords,
+                const std::vector<npy_intp>& non_degenerate) const {
+    CubeID id = 0;
+    for (int n = ndim_ - 1; n >= 0; --n) {
+      id = (id << required_bits_[n]) | (coords[n] << 1) | non_degenerate[n];
+    }
+    return id;
   }
 
-  std::vector<CubeID> Facets(CubeID id) {
+  void Decode(CubeID id,
+              std::vector<npy_intp>* coords,
+              std::vector<npy_intp>* non_degenerate) const {
+    for (int n = 0; n < ndim_; ++n) {
+      CubeID slice = id & Mask(n);
+      coords->at(n) = slice >> 1;
+      non_degenerate->at(n) = slice & 1;
+      id >>= required_bits_[n];
+    }
+  }
+
+  CubeID Mask(int n) const {
+    return ((1 << required_bits_[n]) - 1);
+  }
+
+  CubeID BuildFacet(CubeID partial, CubeID coface, CubeID mask, int n) const {
+    return (coface & ~mask) | (partial << (bottom_bit_[n] + 1));
+  }
+
+  std::vector<CubeID> Facets(CubeID id) const {
     std::vector<CubeID> result;
 
     for (int n = 0; n < ndim_; ++n) {
-      CubeID mask = Mask(n) << bottom_bit_->at(n);
-      CubeID slice = (id & mask) >> bottom_bit_->at(n);
+      CubeID mask = Mask(n) << bottom_bit_[n];
+      CubeID slice = (id & mask) >> bottom_bit_[n];
       if (slice & 1) {
         result.push_back(BuildFacet(((slice >> 1) + 1) % shape_[n], id, mask, n));
         result.push_back(BuildFacet(slice >> 1, id, mask, n));
@@ -145,6 +115,135 @@ struct CubicalFiltrationExt {
     return result;
   }
 
+  size_t TotalBits() const {
+    return bottom_bit_[ndim_];
+  }
+
+  // For Python
+  static bool ValidSequence(PyObject* obj, int expected_size) {
+    return PySequence_Check(obj) && (PySequence_Size(obj) == expected_size);
+  }
+
+  PyObject* encode_cube(PyObject* args) const {
+    PyObject* coords;
+    PyObject* non_degenerate;
+
+    if (!PyArg_ParseTuple(args, "OO", &coords, &non_degenerate))
+      return nullptr;
+
+    if (!ValidSequence(coords, ndim_) ||
+        !ValidSequence(non_degenerate, ndim_)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "coords or non_degenerate is not a valid sequence");
+      return nullptr;
+    }
+
+    std::vector<npy_intp> v_coords(ndim_);
+    std::vector<npy_intp> v_non_degenerate(ndim_);
+    for (int n = 0; n < ndim_; ++n) {
+      v_coords[n] = PyLong_AsLong(PySequence_GetItem(coords, n));
+      v_non_degenerate[n] = PyLong_AsLong(PySequence_GetItem(non_degenerate, n));
+    }
+    return PyLong_FromLong(Encode(v_coords, v_non_degenerate));
+  }
+
+  PyObject* decode_cube(PyObject* args) const {
+    CubeID id;
+    if (!PyArg_ParseTuple(args, "K", &id))
+      return nullptr;
+
+    std::vector<npy_intp> v_coords(ndim_);
+    std::vector<npy_intp> v_non_degenerate(ndim_);
+
+    Decode(id, &v_coords, &v_non_degenerate);
+
+    PyObject* coords = nullptr;
+    PyObject* non_degenerate = nullptr;
+
+    coords = PyList_New(ndim_);
+    non_degenerate = PyList_New(ndim_);
+    if (!coords || !non_degenerate)
+      goto error;
+
+    for (int n = 0; n < ndim_; ++n) {
+      PyObject* coord = PyLong_FromLong(v_coords[n]);
+      if (!coord) goto error;
+      PyList_SET_ITEM(coords, n, coord);
+    
+      PyObject* ndeg = PyLong_FromLong(v_non_degenerate[n]);
+      if (!ndeg) goto error;
+      PyList_SET_ITEM(non_degenerate, n, ndeg);
+    }
+
+    return Py_BuildValue("(OO)", coords, non_degenerate);
+
+ error:
+    Py_XDECREF(coords);
+    Py_XDECREF(non_degenerate);
+    return nullptr;
+  }
+
+  PyObject* cube_dim(PyObject* args) {
+    CubeID id;
+    if (!PyArg_ParseTuple(args, "K", &id))
+      return nullptr;
+
+    return PyLong_FromLong(CubeDim(id));
+  }
+
+  int ndim_;
+  std::vector<npy_intp> shape_;
+  std::vector<npy_intp> required_bits_;
+  std::vector<npy_intp> bottom_bit_;
+  CubeID cubedim_mask_;
+}; // struct CubeEncoder
+
+struct CubeEncoderClass {
+  PyObject_HEAD
+  CubeEncoder* encoder_;
+};
+
+struct CubicalFiltrationExt {
+  PyObject_HEAD
+  PyArrayObject* array_;
+  PyObject* periodicity_;
+  int ndim_;
+  npy_intp* shape_;
+  CubeEncoder* encoder_;
+  std::vector<double>* cache_value_at_;
+  std::vector<CubeID>* sorted_cubes_;
+  bool save_boundary_map_;
+
+  void SetUp(PyArrayObject* array, PyObject* periodicity, bool save_boundary_map) {
+    ndim_ = PyArray_NDIM(array);
+    shape_ = PyArray_DIMS(array);
+    array_ = array;
+    periodicity_ = periodicity;
+    Py_INCREF(array_);
+    Py_INCREF(periodicity_);
+
+    encoder_ = new CubeEncoder(ndim_, shape_);
+    
+    cache_value_at_ = new std::vector<double>(1 << encoder_->bottom_bit_[ndim_],
+                                             std::numeric_limits<double>::quiet_NaN());
+
+    PrepareSortedCubes();
+
+    save_boundary_map_ = save_boundary_map;
+  }
+
+  void TearDown() {
+    Py_XDECREF(array_);
+    Py_XDECREF(periodicity_);
+    delete encoder_; encoder_ = nullptr;
+    delete cache_value_at_; cache_value_at_ = nullptr;
+    delete sorted_cubes_; sorted_cubes_ = nullptr;
+  }
+
+  int CubeDim(CubeID id) {
+    return encoder_->CubeDim(id);
+  }
+
   std::vector<int> BoundarySigns(CubeID id) {
     int dim = CubeDim(id);
     std::vector<int> result(2 * dim);
@@ -162,7 +261,7 @@ struct CubicalFiltrationExt {
     if (CubeDim(id) == 0) {
       std::vector<npy_intp> coords(ndim_);
       std::vector<npy_intp> non_degenerate(ndim_);
-      Decode(id, &coords, &non_degenerate);
+      encoder_->Decode(id, &coords, &non_degenerate);
       value = *reinterpret_cast<double*>(PyArray_GetPtr(array_, coords.data()));
     } else {
       auto facets = Facets(id);
@@ -174,10 +273,14 @@ struct CubicalFiltrationExt {
     return value;
   }
 
+  std::vector<CubeID> Facets(CubeID id) {
+    return encoder_->Facets(id);
+  }
+
   bool ValidCoords(const std::vector<npy_intp>& coords,
                    const std::vector<npy_intp>& non_degenerate) const {
     for (int n = 0; n < ndim_; ++n) {
-      if (PyObject_IsTrue(PySequence_GetItem(periodic_, n)))
+      if (PyObject_IsTrue(PySequence_GetItem(periodicity_, n)))
         continue;
       if (coords[n] + non_degenerate[n] >= shape_[n])
         return false;
@@ -190,7 +293,7 @@ struct CubicalFiltrationExt {
                     std::vector<npy_intp>* non_degenerate,
                     std::vector<CubeID>* result) {
     if (n == ndim_) {
-      CubeID cubeid = Encode(*indexlist, *non_degenerate);
+      CubeID cubeid = encoder_->Encode(*indexlist, *non_degenerate);
       if (ValidCoords(*indexlist, *non_degenerate) && ValueAt(cubeid) != INFINITY) {
         result->push_back(cubeid);
       }
@@ -235,57 +338,13 @@ struct CubicalFiltrationExt {
   }
 
   std::vector<int64_t> Cube2Index() {
-    std::vector<int64_t> cube2index(1 << bottom_bit_->at(ndim_));
+    std::vector<int64_t> cube2index(1 << encoder_->TotalBits());
     for (unsigned n = 0; n < sorted_cubes_->size(); ++n)
       cube2index[(*sorted_cubes_)[n]] = n;
     return cube2index;
   }
 
-  std::string DiphaByteSequence() {
-    std::string buf;
-    StringWriter output(&buf);
-
-    PrepareSortedCubes();
-    std::vector<int64_t> cube2index = Cube2Index();
-
-    // Write headers
-    output.Write<int64_t>(8067171840);
-    output.Write<int64_t>(0);
-    output.Write<int64_t>(0);
-    output.Write<int64_t>(sorted_cubes_->size());
-    output.Write<int64_t>(ndim_);
-    
-    // Write dimensions
-    for (CubeID cube : *sorted_cubes_)
-      output.Write<int64_t>(CubeDim(cube));
-
-    // Write birth time
-    if (indexized_) {
-      for (unsigned i = 0; i < sorted_cubes_->size(); ++i)
-        output.Write<double>(i);
-    } else {
-      for (CubeID cube : *sorted_cubes_)
-        output.Write<double>(ValueAt(cube));
-    }
-
-    // Write Boundary map sizes
-    int n = 0;
-    for (CubeID cube : *sorted_cubes_) {
-      output.Write<int64_t>(n);
-      n += CubeDim(cube) * 2;
-    }
-    output.Write<int64_t>(n);
-
-    // Write boundary map
-    for (CubeID cube : *sorted_cubes_)
-      for (CubeID facet : Facets(cube))
-        output.Write<int64_t>(cube2index[facet]);
-
-    return buf;
-  }
-
   phat_ext::Matrix* BuildPhatMatrix() {
-    PrepareSortedCubes();
     phat_ext::Matrix* matrix = phat_ext::MatrixNew(
         sorted_cubes_->size(), save_boundary_map_ ? "cubical" : "none");
         
@@ -306,6 +365,7 @@ struct CubicalFiltrationExt {
   }
 }; // struct CubicalFiltrationExt
 
+
 namespace CubicalFiltrationExt_methods {
 
 static void dealloc(CubicalFiltrationExt* self) {
@@ -315,25 +375,21 @@ static void dealloc(CubicalFiltrationExt* self) {
 
 static int init(CubicalFiltrationExt* self, PyObject* args, PyObject* kwds) {
   PyArrayObject* array;
-  PyObject* periodic;
+  PyObject* periodicity;
   int save_boundary_map;
   
   if (!PyArg_ParseTuple(args, "O!O!p", &PyArray_Type, &array,
-                        &PyList_Type, &periodic, &save_boundary_map))
+                        &PyList_Type, &periodicity, &save_boundary_map))
     return -1;
 
   if (!ArrayIsDoubleType(array))
     return -1;
 
-  self->SetUp(array, periodic, save_boundary_map);
+  self->SetUp(array, periodicity, save_boundary_map);
 
   return 0;
 }
 
-static bool ValidSequence(PyObject* obj, int expected_size) {
-  return PySequence_Check(obj) && (PySequence_Size(obj) == expected_size);
-}
-
 static PyObject* get_required_bits(PyObject* self_, void* closure) {
   CubicalFiltrationExt* self = reinterpret_cast<CubicalFiltrationExt*>(self_);
   PyObject* list = PyList_New(self->ndim_);
@@ -342,7 +398,7 @@ static PyObject* get_required_bits(PyObject* self_, void* closure) {
     return nullptr;
 
   for (int n = 0; n < self->ndim_; ++n) {
-    PyObject* bits = PyLong_FromLong(self->required_bits_->at(n));
+    PyObject* bits = PyLong_FromLong(self->encoder_->required_bits_[n]);
     if (!bits)
       goto error;
 
@@ -356,7 +412,6 @@ error:
 }
 
 static PyObject* get_sorted_cubes(CubicalFiltrationExt* self, void* closure) {
-  self->PrepareSortedCubes();
   PyObject* list = PyList_New(self->sorted_cubes_->size());
   
   if (!list)
@@ -375,73 +430,18 @@ error:
 }
 
 static PyObject* encode_cube(CubicalFiltrationExt* self, PyObject* args) {
-  PyObject* coords;
-  PyObject* non_degenerate;
-
-  if (!PyArg_ParseTuple(args, "OO", &coords, &non_degenerate))
-    return nullptr;
-
-  if (!ValidSequence(coords, self->ndim_) ||
-      !ValidSequence(non_degenerate, self->ndim_)) {
-    PyErr_SetString(PyExc_RuntimeError,
-                    "coords or non_degenerate is not a valid sequence");
-    return nullptr;
-  }
-
-  std::vector<npy_intp> v_coords(self->ndim_);
-  std::vector<npy_intp> v_non_degenerate(self->ndim_);
-  for (int n = 0; n < self->ndim_; ++n) {
-    v_coords[n] = PyLong_AsLong(PySequence_GetItem(coords, n));
-    v_non_degenerate[n] = PyLong_AsLong(PySequence_GetItem(non_degenerate, n));
-  }
-  return PyLong_FromLong(self->Encode(v_coords, v_non_degenerate));
+  return self->encoder_->encode_cube(args);
 }
 
 static PyObject* decode_cube(CubicalFiltrationExt* self, PyObject* args) {
-  CubeID id;
-  if (!PyArg_ParseTuple(args, "K", &id))
-    return nullptr;
-
-  std::vector<npy_intp> v_coords(self->ndim_);
-  std::vector<npy_intp> v_non_degenerate(self->ndim_);
-
-  self->Decode(id, &v_coords, &v_non_degenerate);
-
-  PyObject* coords = nullptr;
-  PyObject* non_degenerate = nullptr;
-
-  coords = PyList_New(self->ndim_);
-  non_degenerate = PyList_New(self->ndim_);
-  if (!coords || !non_degenerate)
-    goto error;
-
-  for (int n = 0; n < self->ndim_; ++n) {
-    PyObject* coord = PyLong_FromLong(v_coords[n]);
-    if (!coord) goto error;
-    PyList_SET_ITEM(coords, n, coord);
-    
-    PyObject* ndeg = PyLong_FromLong(v_non_degenerate[n]);
-    if (!ndeg) goto error;
-    PyList_SET_ITEM(non_degenerate, n, ndeg);
-  }
-
-  return Py_BuildValue("(OO)", coords, non_degenerate);
-
-error:
-  Py_XDECREF(coords);
-  Py_XDECREF(non_degenerate);
-  return nullptr;
+  return self->encoder_->decode_cube(args);
 }
 
-PyObject* cube_dim(CubicalFiltrationExt* self, PyObject* args) {
-  CubeID id;
-  if (!PyArg_ParseTuple(args, "K", &id))
-    return nullptr;
-
-  return PyLong_FromLong(self->CubeDim(id));
+static PyObject* cube_dim(CubicalFiltrationExt* self, PyObject* args) {
+  return self->encoder_->cube_dim(args);
 }
 
-PyObject* boundary(CubicalFiltrationExt* self, PyObject* args) {
+static PyObject* boundary(CubicalFiltrationExt* self, PyObject* args) {
   CubeID id;
   if (!PyArg_ParseTuple(args, "K", &id))
     return nullptr;
@@ -468,7 +468,7 @@ PyObject* boundary(CubicalFiltrationExt* self, PyObject* args) {
   return Py_BuildValue("(OO)", facets_list, signs_list);
 }
 
-PyObject* value_at(CubicalFiltrationExt* self, PyObject* args) {
+static PyObject* value_at(CubicalFiltrationExt* self, PyObject* args) {
   CubeID id;
   if (!PyArg_ParseTuple(args, "K", &id))
     return nullptr;
@@ -476,7 +476,7 @@ PyObject* value_at(CubicalFiltrationExt* self, PyObject* args) {
   return PyFloat_FromDouble(self->ValueAt(id));
 }
 
-PyObject* all_cubes(CubicalFiltrationExt* self) {
+static PyObject* all_cubes(CubicalFiltrationExt* self) {
   PyObject* list = PyList_New(0);
   PyObject* cubeint = nullptr;
 
@@ -495,28 +495,14 @@ error:
   return nullptr;
 }
 
-PyObject* dipha_byte_sequence(CubicalFiltrationExt* self) {
-  std::string buf = self->DiphaByteSequence();
-  return PyBytes_FromStringAndSize(buf.c_str(), buf.size());
-}
-
-PyObject* indexize_internal(CubicalFiltrationExt* self) {
-  if (self->indexized_)
-    Py_RETURN_NONE;
-
-  self->PrepareSortedCubes();
-
-  npy_intp dims[]= {static_cast<npy_intp>(self->sorted_cubes_->size())};
+static PyObject* levels(CubicalFiltrationExt* self) {
+  npy_intp dims[] = {static_cast<npy_intp>(self->sorted_cubes_->size())};
   PyObject* levels = PyArray_SimpleNew(1, dims, NPY_DOUBLE);
-
   if (!levels)
     return nullptr;
 
   for (unsigned i = 0; i < self->sorted_cubes_->size(); ++i)
     *GETPTR1D<double>((PyArrayObject*)levels, i) = self->ValueAt((*self->sorted_cubes_)[i]);
-
-  self->indexized_ = true;
-
   return levels;
 }
 
@@ -537,10 +523,7 @@ static PyMethodDef methods[] = {
   {"boundary", (PyCFunction)boundary, METH_VARARGS, "Boundary map"},
   {"value_at", (PyCFunction)value_at, METH_VARARGS, "Level of the cube"},
   {"all_cubes", (PyCFunction)all_cubes, METH_NOARGS, "List of all cubes"},
-  {"dipha_byte_sequence", (PyCFunction)dipha_byte_sequence, METH_NOARGS,
-   "Byte sequence for dipha"},
-  {"indexize_internal", (PyCFunction)indexize_internal, METH_NOARGS,
-   "Indexize internally"},
+  {"levels", (PyCFunction)levels, METH_NOARGS, "index to levels"},
   {"build_phat_matrix", (PyCFunction)build_phat_matrix, METH_NOARGS,
    "Return phat' matrix object"},
   {NULL}
@@ -549,6 +532,8 @@ static PyMethodDef methods[] = {
 static PyMemberDef members[] = {
   {"array", T_OBJECT, offsetof(CubicalFiltrationExt, array_), READONLY,
    "Numpy array object"},
+  {"periodicity", T_OBJECT, offsetof(CubicalFiltrationExt, periodicity_),
+   READONLY, "periodicity"},
   {NULL}
 };
 
@@ -602,6 +587,97 @@ static PyTypeObject CubicalFiltrationExtType = {
   PyType_GenericNew, // tp_new
 }; 
 
+namespace CubeEncoderClass_methods {
+
+static int init(CubeEncoderClass* self, PyObject* args, PyObject* kwds) {
+  PyObject* shape;
+  
+  if (!PyArg_ParseTuple(args, "O", &shape))
+    return -1;
+
+  if (!PySequence_Check(shape))
+    return -1;
+
+  int ndim = PySequence_Size(shape);
+  std::vector<npy_intp> vshape(ndim);
+  for (int n = 0; n < ndim; ++n) {
+    vshape[n] = PyLong_AsLong(PySequence_GetItem(shape, n));
+    if (PyErr_Occurred())
+      return -1;
+  }
+
+  self->encoder_ = new CubeEncoder(ndim, vshape.data());
+  return 0;
+}
+
+static void dealloc(CubeEncoderClass* self) {
+  delete self->encoder_;
+  Py_TYPE(self)->tp_free(cast_PyObj(self));
+}
+
+static PyObject* encode_cube(CubeEncoderClass* self, PyObject* args) {
+  return self->encoder_->encode_cube(args);
+}
+
+static PyObject* decode_cube(CubeEncoderClass* self, PyObject* args) {
+  return self->encoder_->decode_cube(args);
+}
+
+static PyObject* cube_dim(CubeEncoderClass* self, PyObject* args) {
+  return self->encoder_->cube_dim(args);
+}
+
+static PyMethodDef methods[] = {
+  {"encode_cube", (PyCFunction)encode_cube, METH_VARARGS, "Encode cube id"},
+  {"decode_cube", (PyCFunction)decode_cube, METH_VARARGS, "Decode cube id"},
+  {"cube_dim", (PyCFunction)cube_dim, METH_VARARGS, "Dimension of the cube id"},
+  {NULL}
+};
+  
+}; // namespace CubeEncoderClass
+
+static PyTypeObject CubeEncoderClassType = {
+  PyVarObject_HEAD_INIT(NULL, 0)
+  "homcloud.cubical_ext.CubeEncoder", // tp_name
+  sizeof(CubeEncoderClass), // tp_basicsize
+  0, // tp_itemsize
+  (destructor)CubeEncoderClass_methods::dealloc, // tp_dealloc
+  0, // tp_print
+  0, // tp_getattr
+  0, // tp_setattr
+  0, // tp_reserved
+  0, // tp_repr
+  0, // tp_as_number
+  0, // tp_as_sequence
+  0, // tp_as_mapping
+  0, // tp_hash 
+  0, // tp_call
+  0, // tp_str
+  0, // tp_getattro
+  0, // tp_setattro
+  0, // tp_as_buffer
+  Py_TPFLAGS_DEFAULT, // tp_flags 
+  "Class for encode/decode cubes of homcloud.cubical_ext.CubicalFiltrationExt",  // tp_doc 
+  0, // tp_traverse
+  0, // tp_clear
+  0, // tp_richcompare
+  0, // tp_weaklistoffset
+  0, // tp_iter
+  0, // tp_iternext
+  CubeEncoderClass_methods::methods,  // tp_methods 
+  0, // tp_members
+  0, // tp_getset
+  0, // tp_base
+  0, // tp_dict
+  0, // tp_descr_get
+  0, // tp_descr_set
+  0, // tp_dictoffset
+  reinterpret_cast<initproc>(CubeEncoderClass_methods::init), // tp_init
+  0, // tp_alloc
+  PyType_GenericNew, // tp_new
+}; 
+
+
 static PyModuleDef cubical_ext_Module = {
   PyModuleDef_HEAD_INIT,
   "homcloud.cubical_ext",
@@ -615,6 +691,8 @@ PyInit_cubical_ext()
 {
   if (PyType_Ready(&CubicalFiltrationExtType) < 0)
     return nullptr;
+  if (PyType_Ready(&CubeEncoderClassType) < 0)
+    return nullptr;
 
   PyObject* module = PyModule_Create(&cubical_ext_Module);
   if (!module)
@@ -624,6 +702,10 @@ PyInit_cubical_ext()
   PyModule_AddObject(module, "CubicalFiltrationExt",
                      cast_PyObj(&CubicalFiltrationExtType));
 
+  Py_INCREF(&CubeEncoderClassType);
+  PyModule_AddObject(module, "CubeEncoder",
+                     cast_PyObj(&CubeEncoderClassType));
+  
   import_array();
   phat_ext::import();
 
