@@ -4,8 +4,6 @@
 #include "homcloud_cgal.h"
 #include "homcloud_numpy.h"
 
-#ifdef CGAL_NEWER_API_4_11
-
 #include "alpha_shape_common.h"
 
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
@@ -97,12 +95,6 @@ typedef CGAL::Dispatch_output_iterator<CGAL::cpp11::tuple<Object, Alpha_value_ty
                                        CGAL::cpp11::tuple<std::back_insert_iterator<std::vector<Object> >,\
                                        std::back_insert_iterator<std::vector<Alpha_value_type> > > > Dispatch;
 
-#if PY_MAJOR_VERSION >= 3
-#define INIT_ERROR return NULL
-#else
-#define INIT_ERROR return
-#endif
-
 typedef struct {
   PyObject_HEAD
   CGAL_AlphaShape3* alpha_shape_;
@@ -199,10 +191,12 @@ static int PeriodicAlphaShape3_init(PeriodicAlphaShape3* self, PyObject* args, P
   if (!ArrayIsDoubleType(points))
     return -1;
 
+#if (CGAL_VERSION_NR < 1050601000)
   if (xmax - xmin != ymax - ymin || xmax - xmin != zmax - zmin) {
-    PyErr_SetString(PyExc_ValueError, "Periodic condition must be cubical.");
+    PyErr_SetString(PyExc_ValueError, "Noncubic periodic condition is only supported CGAL >= 5.6.");
     return -1;
   }
+#endif
 
   npy_intp* dims = PyArray_DIMS(points);
 
@@ -798,7 +792,20 @@ static PyTypeObject PeriodicAlphaShape3Type = {
 
 /////////////// Vertex ///////////////////////////
 
+
+static int Vertex_traverse(Vertex* vertex, visitproc visit, void* arg) {
+  Py_VISIT(vertex->alpha_);
+  return 0;
+}
+
+static int Vertex_clear(Vertex* vertex) {
+  Py_CLEAR(vertex->alpha_);
+  return 0;
+}
+
 static void Vertex_dealloc(Vertex* self) {
+  PyObject_GC_UnTrack(self);
+  Vertex_clear(self);
   Py_TYPE(self)->tp_free(cast_PyObj(self));
 }
 
@@ -881,10 +888,10 @@ static PyTypeObject VertexType = {
   0, // tp_getattro
   0, // tp_setattro
   0, // tp_as_buffer
-  Py_TPFLAGS_DEFAULT, // tp_flags 
+  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC, // tp_flags 
   "Vertex class for PeriodicAlphaShape3",  // tp_doc
-  0, // tp_traverse
-  0, // tp_clear
+  (traverseproc)Vertex_traverse, // tp_traverse
+  (inquiry)Vertex_clear, // tp_clear
   Vertex_compare, // tp_richcompare
   0, // tp_weaklistoffset
   0, // tp_iter
@@ -895,7 +902,8 @@ static PyTypeObject VertexType = {
 
 static Vertex* Vertex_New(PeriodicAlphaShape3* alpha,
                           CGAL_AlphaShape3::Vertex_handle handle) {
-  Vertex* vertex = PyObject_New(Vertex, &VertexType);
+  Vertex* vertex = PyObject_GC_New(Vertex, &VertexType);
+
   if (!vertex) return NULL;
 
   vertex->alpha_ = alpha;
@@ -904,6 +912,7 @@ static Vertex* Vertex_New(PeriodicAlphaShape3* alpha,
   vertex->group_name_ = handle->group_name;
   vertex->birth_radius_ = -(handle->point().weight());
   Py_INCREF(alpha);
+  PyObject_GC_Track(vertex);
 
   return vertex;
 }
@@ -918,7 +927,18 @@ static bool Vertex_equal(PyObject* o1, PyObject* o2) {
 
 /////////////// Cell ///////////////////////////
 
+static int Cell_traverse(Cell* cell, visitproc visit, void* arg) {
+  Py_VISIT(cell->alpha_);
+  return 0;
+}
+static int Cell_clear(Cell* cell) {
+  Py_CLEAR(cell->alpha_);
+  return 0;
+}
+
 static void Cell_dealloc(Cell* self) {
+  PyObject_GC_UnTrack(self);
+  Cell_clear(self);
   Py_TYPE(self)->tp_free(cast_PyObj(self));
 }
 
@@ -987,10 +1007,10 @@ static PyTypeObject CellType = {
   0, // tp_getattro
   0, // tp_setattro
   0, // tp_as_buffer
-  Py_TPFLAGS_DEFAULT, // tp_flags 
+  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC, // tp_flags 
   "Cell class for PeriodicAlphaShape3",  // tp_doc
-  0, // tp_traverse
-  0, // tp_clear  
+  (traverseproc)Cell_traverse, // tp_traverse
+  (inquiry)Cell_clear, // tp_clear
   Cell_compare, // tp_richcompare
   0, // tp_weaklistoffset
   0, // tp_iter
@@ -1000,13 +1020,14 @@ static PyTypeObject CellType = {
 };
 
 static Cell* Cell_New(PeriodicAlphaShape3* alpha, CGAL_AlphaShape3::Cell_handle handle) {
-  Cell* cell = PyObject_New(Cell, &CellType);
+  Cell* cell = PyObject_GC_New(Cell, &CellType);
   if (!cell) return NULL;
 
   cell->alpha_ = alpha;
   cell->handle_ = handle;
   cell->birth_radius_ = AlphaValueCell(alpha->alpha_shape_,handle);
   Py_INCREF(alpha);
+  PyObject_GC_Track(cell);
 
   return cell;
 }
@@ -1021,8 +1042,19 @@ static bool Cell_equal(PyObject* o1, PyObject* o2) {
 
 /////////////// Edge ///////////////////////////
 
+static int Edge_traverse(Edge* edge, visitproc visit, void* arg) {
+  Py_VISIT(edge->alpha_);
+  return 0;
+}
+
+static int Edge_clear(Edge* edge) {
+  Py_CLEAR(edge->alpha_);
+  return 0;
+}
+
 static void Edge_dealloc(Edge* self) {
-  Py_XDECREF(self->alpha_);
+  PyObject_GC_UnTrack(self);
+  Edge_clear(self);
   Py_TYPE(self)->tp_free(cast_PyObj(self));
 }
 
@@ -1076,10 +1108,10 @@ static PyTypeObject EdgeType = {
   0, // tp_getattro
   0, // tp_setattro
   0, // tp_as_buffer
-  Py_TPFLAGS_DEFAULT, // tp_flags 
+  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC, // tp_flags 
   "Edge class for PeriodicAlphaShape3", // tp_doc
-  0, // tp_traverse
-  0, // tp_clear
+  (traverseproc)Edge_traverse, // tp_traverse
+  (inquiry)Edge_clear, // tp_clear
   0, // tp_richcompare 
   0, // tp_weaklistoffset
   0, // tp_iter
@@ -1090,7 +1122,7 @@ static PyTypeObject EdgeType = {
 
 static Edge* Edge_New(PeriodicAlphaShape3* alpha,
                       CGAL_AlphaShape3::Edge &e) {
-  Edge* edge = PyObject_New(Edge, &EdgeType);
+  Edge* edge = PyObject_GC_New(Edge, &EdgeType);
   if (!edge) return NULL;
 
   // e.first : Cell_handle
@@ -1102,14 +1134,27 @@ static Edge* Edge_New(PeriodicAlphaShape3* alpha,
   edge->birth_radius_ = AlphaValueEdge(alpha->alpha_shape_,&e);
 
   Py_INCREF(alpha);
+  PyObject_GC_Track(edge);
 
   return edge;
 }
 
 /////////////// Facet ///////////////////////////
 
+
+static int Facet_traverse(Facet* facet, visitproc visit, void* arg) {
+  Py_VISIT(facet->alpha_);
+  return 0;
+}
+
+static int Facet_clear(Facet* facet) {
+  Py_CLEAR(facet->alpha_);
+  return 0;
+}
+
 static void Facet_dealloc(Facet* self) {
-  Py_XDECREF(self->alpha_);
+  PyObject_GC_UnTrack(self);
+  Facet_clear(self);
   Py_TYPE(self)->tp_free(cast_PyObj(self));
 }
 
@@ -1163,10 +1208,10 @@ static PyTypeObject FacetType = {
   0, // tp_getattro
   0, // tp_setattro
   0, // tp_as_buffer
-  Py_TPFLAGS_DEFAULT, // tp_flags 
+  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC, // tp_flags 
   "Facet class for PeriodicAlphaShape3", // tp_doc
-  0, // tp_traverse
-  0, // tp_clear
+  (traverseproc)Facet_traverse, // tp_traverse
+  (inquiry)Facet_clear, // tp_clear
   0, // tp_richcompare 
   0, // tp_weaklistoffset
   0, // tp_iter
@@ -1177,7 +1222,7 @@ static PyTypeObject FacetType = {
 
 static Facet* Facet_New(PeriodicAlphaShape3* alpha,
                         CGAL_AlphaShape3::Facet &f) {
-  Facet* facet = PyObject_New(Facet, &FacetType);
+  Facet* facet = PyObject_GC_New(Facet, &FacetType);
   if (!facet) return NULL;
 
   facet->alpha_ = alpha;
@@ -1185,47 +1230,38 @@ static Facet* Facet_New(PeriodicAlphaShape3* alpha,
   facet->v_ = f.second;
   facet->birth_radius_ = AlphaValueFacet(alpha->alpha_shape_,&f);
   Py_INCREF(alpha);
+  PyObject_GC_Track(facet);
 
   return facet;
 }
 
 /////////////// Module ///////////////////////////
 
-#if PY_MAJOR_VERSION >= 3
 static PyModuleDef periodic_alpha_shape3_Module = {
   PyModuleDef_HEAD_INIT,
   "homcloud.periodic_alpha_shape3",
   "The module for 3D Periodic Alpha shape",
   -1,
 };
-#endif
 
-#if PY_MAJOR_VERSION >= 3
 PyMODINIT_FUNC
 PyInit_periodic_alpha_shape3()
-#else
-extern "C" void initperiodic_alpha_shape3()
-#endif
 {
   if (PyType_Ready(&PeriodicAlphaShape3Type) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&VertexType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&CellType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&EdgeType) < 0)
-    INIT_ERROR;
+    return NULL;
   if (PyType_Ready(&FacetType) < 0)
-    INIT_ERROR;
+    return NULL;
 
-#if PY_MAJOR_VERSION >= 3
   PyObject* module = PyModule_Create(&periodic_alpha_shape3_Module);
-#else
-  PyObject* module = Py_InitModule("homcloud.periodic_alpha_shape3", NULL);
-#endif
 
   if (!module)
-    INIT_ERROR;
+    return NULL;
 
   Py_INCREF(&PeriodicAlphaShape3Type);
   PyModule_AddObject(module, "PeriodicAlphaShape3", cast_PyObj(&PeriodicAlphaShape3Type));
@@ -1240,85 +1276,5 @@ extern "C" void initperiodic_alpha_shape3()
 
   import_array();
 
-#if PY_MAJOR_VERSION >= 3
-  return module;
-#endif
-}
-
-#else  // CGAL_NEWER_API_4_11
-
-typedef struct {
-  PyObject_HEAD
-} PeriodicAlphaShape3;
-
-static int PeriodicAlphaShape3_init(PeriodicAlphaShape3* self, PyObject* args, PyObject* kwds) {
-  PyErr_SetString(PyExc_RuntimeError, "Periodic Alpha filtration is not supported old CGAL");
-  return -1;
-}
-
-static PyTypeObject PeriodicAlphaShape3Type = {
-  PyVarObject_HEAD_INIT(NULL, 0)
-  "homcloud.periodic_alpha_shape3.PeriodicAlphaShape3", // tp_name
-  sizeof(PeriodicAlphaShape3), // tp_basicsize 
-  0, // tp_itemsize
-  0, // tp_dealloc
-  0, // tp_print
-  0, // tp_getattr
-  0, // tp_setattr
-  0, // tp_reserved
-  0, // tp_repr
-  0, // tp_as_number
-  0, // tp_as_sequence
-  0, // tp_as_mapping
-  0, // tp_hash 
-  0, // tp_call
-  0, // tp_str
-  0, // tp_getattro
-  0, // tp_setattro
-  0, // tp_as_buffer
-  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, // tp_flags 
-  "Dummy Class for old CGAL (< 4.11)",  // tp_doc 
-  0, // tp_traverse
-  0, // tp_clear
-  0, // tp_richcompare
-  0, // tp_weaklistoffset
-  0, // tp_iter
-  0, // tp_iternext
-  0, // tp_methods 
-  0, // tp_members
-  0, // tp_getset
-  0, // tp_base
-  0, // tp_dict
-  0, // tp_descr_get
-  0, // tp_descr_set
-  0, // tp_dictoffset
-  reinterpret_cast<initproc>(PeriodicAlphaShape3_init), // tp_init 
-  0, // tp_alloc
-  PyType_GenericNew, // tp_new
-}; 
-
-static PyModuleDef periodic_alpha_shape3_Module = {
-  PyModuleDef_HEAD_INIT,
-  "homcloud.periodic_alpha_shape3",
-  "The module for 3D Periodic Alpha shape (dummy)",
-  -1,
-};
-
-PyMODINIT_FUNC
-PyInit_periodic_alpha_shape3()
-{
-  if (PyType_Ready(&PeriodicAlphaShape3Type) < 0)
-    return nullptr;
-
-  PyObject* module = PyModule_Create(&periodic_alpha_shape3_Module);
-
-  if (!module)
-    return nullptr;
-
-  Py_INCREF(&PeriodicAlphaShape3Type);
-  PyModule_AddObject(module, "PeriodicAlphaShape3", cast_PyObj(&PeriodicAlphaShape3Type));
-
   return module;
 }
-
-#endif  // CGAL_NEWER_API_4_11
