@@ -1,47 +1,54 @@
 import math
-import struct
 from collections import defaultdict
 from operator import attrgetter
 
-import numpy as np
 from cached_property import cached_property
 import msgpack
 
 from homcloud.alpha_shape2 import AlphaShape2 as AlphaShape2Base
 from homcloud.alpha_shape3 import AlphaShape3 as AlphaShape3Base
+from homcloud.periodic_alpha_shape2 import PeriodicAlphaShape2 as PeriodicAlphaShape2Base
 from homcloud.periodic_alpha_shape3 import PeriodicAlphaShape3 as PeriodicAlphaShape3Base
-from homcloud.index_map import IndexMapForAlphaFiltration
-from homcloud.base_filtration import FiltrationWithIndex
 from homcloud.pdgm_format import PDGMWriter, BinaryChunk, AlphaInformationChunk
 import homcloud.phat_ext as phat
 import homcloud.build_phtrees as build_phtrees
 
 
-class AlphaShape(object):
+class AlphaShape:
     @staticmethod
     def build(points, dim, weighted=False, use_relative_homology=False, periodicity=None):
         if dim == 2:
-            return AlphaShape2(points, weighted, use_relative_homology)
+            if periodicity is None:
+                return AlphaShape2(points, weighted, use_relative_homology)
+            else:
+                return PeriodicAlphaShape2(points, weighted, use_relative_homology, periodicity)
         if dim == 3:
             if periodicity is None:
                 return AlphaShape3(points, weighted, use_relative_homology)
             else:
-                return PeriodicAlphaShape3(points, weighted, use_relative_homology,
-                                           periodicity)
+                return PeriodicAlphaShape3(points, weighted, use_relative_homology, periodicity)
 
         raise ValueError("dimension of a point cloud must be 2 or 3")
 
-    def create_filtration(self, squared=True, symbols=None, save_boundary_map=False,
-                          save_phtrees=False):
+    def create_filtration(self, squared=True, symbols=None, save_boundary_map=False, save_phtrees=False):
         simplices = [
             Simplex(alpha_simplex, i, squared)
-            for (i, alpha_simplex)
-            in enumerate(sorted(self.simplices, key=attrgetter("birth_radius")))
+            for (i, alpha_simplex) in enumerate(sorted(self.simplices, key=attrgetter("birth_radius")))
         ]
 
         dict_simplices = {simplex.key: simplex for simplex in simplices}
-        return AlphaFiltration(self.coordinates, self.weights, squared, self.periodicity, simplices, dict_simplices,
-                               self.dim, symbols, save_boundary_map, save_phtrees)
+        return AlphaFiltration(
+            self.coordinates,
+            self.weights,
+            squared,
+            self.periodicity,
+            simplices,
+            dict_simplices,
+            self.dim,
+            symbols,
+            save_boundary_map,
+            save_phtrees,
+        )
 
     def subsets(self):
         def group_of_simplex(simplex):
@@ -68,8 +75,7 @@ class AlphaShape(object):
 
         return {
             group_name: AlphaSubset(group_name, self.coordinates, self.weights, self.periodicity, simplices, self.dim)
-            for (group_name, simplices)
-            in group_by(self.simplices, group_of_simplex).items()
+            for (group_name, simplices) in group_by(self.simplices, group_of_simplex).items()
         }
 
     def all_subsets_acyclic(self):
@@ -79,7 +85,7 @@ class AlphaShape(object):
         for subset in self.subsets().values():
             if not subset.isacyclic():
                 message = "Subset {} is not acyclic".format(subset.group_name)
-                raise(RuntimeError(message))
+                raise RuntimeError(message)
 
     def become_partial_shape(self):
         for subset in self.subsets().values():
@@ -110,7 +116,7 @@ class AlphaShape3(AlphaShape3Base, AlphaShape):
 class AlphaShape2(AlphaShape2Base, AlphaShape):
     def __init__(self, points, weighted, rel_homology):
         super().__init__(points, weighted, rel_homology)
-        self.weights = points[:, 3] if weighted else None
+        self.weights = points[:, 2] if weighted else None
         self.coordinates = points[:, 0:2]
         self.simplices = self.vertices() + self.edges() + self.faces()
 
@@ -123,11 +129,26 @@ class AlphaShape2(AlphaShape2Base, AlphaShape):
         return None
 
 
+class PeriodicAlphaShape2(PeriodicAlphaShape2Base, AlphaShape):
+    def __init__(self, points, weighted, rel_homology, periodicity):
+        if weighted:
+            raise ValueError("Weighted 2D alpha shape is not support due to the limitation of CGAL")
+        ((xmin, xmax), (ymin, ymax)) = periodicity
+        super().__init__(points, False, rel_homology, xmin, xmax, ymin, ymax)
+        self.weights = None
+        self.coordinates = points[:, :2]
+        self.simplices = self.vertices() + self.edges() + self.faces()
+        self.periodicity = periodicity
+
+    @property
+    def dim(self):
+        return 2
+
+
 class PeriodicAlphaShape3(PeriodicAlphaShape3Base, AlphaShape):
     def __init__(self, points, weighted, rel_homology, periodicity):
         ((xmin, xmax), (ymin, ymax), (zmin, zmax)) = periodicity
-        super().__init__(points, weighted, rel_homology,
-                         xmin, xmax, ymin, ymax, zmin, zmax)
+        super().__init__(points, weighted, rel_homology, xmin, xmax, ymin, ymax, zmin, zmax)
         self.coordinates = points[:, 0:3]
         self.weights = points[:, 3] if weighted else None
         self.simplices = self.vertices() + self.edges() + self.facets() + self.cells()
@@ -151,7 +172,7 @@ class AlphaSubset(AlphaShape):
         return self.create_filtration().isacyclic()
 
 
-class Simplex(object):
+class Simplex:
     """A class representing simplex"""
 
     def __init__(self, alpha_simplex, index, squared=True):
@@ -169,7 +190,7 @@ class Simplex(object):
         """Return list of frozensets of vertices of indices of boundary simplices"""
         if self.isvertex:
             return []
-        return [self.key[0:n] + self.key[n + 1:] for n in range(len(self.key))]
+        return [self.key[0:n] + self.key[n + 1 :] for n in range(len(self.key))]
 
     def signed_boundary_keys(self):
         def plusminus_alternative(length):
@@ -188,9 +209,20 @@ class Simplex(object):
         return len(self.key) - 1
 
 
-class AlphaFiltration(FiltrationWithIndex):
-    def __init__(self, points, point_weights, squared, periodicity, simplices, dict_simplices, dim, symbols,
-                 save_boundary_map, save_phtrees):
+class AlphaFiltration:
+    def __init__(
+        self,
+        points,
+        point_weights,
+        squared,
+        periodicity,
+        simplices,
+        dict_simplices,
+        dim,
+        symbols,
+        save_boundary_map,
+        save_phtrees,
+    ):
         """
         Args:
         points -- list of N-d point coordinates
@@ -216,13 +248,15 @@ class AlphaFiltration(FiltrationWithIndex):
         self.save_phtrees = save_phtrees
 
     def alpha_information_chunk(self):
-        return AlphaInformationChunk(self.points.shape[0], self.periodicity,
-                                     self.points_weighted, self.squared)
+        return AlphaInformationChunk(self.points.shape[0], self.periodicity, self.points_weighted, self.squared)
 
-    def compute_pdgm(self, f, algorithm=None, output_suppl_info=True):
+    def compute_pdgm(self, f, algorithm=None, output_suppl_info=True, parallels=None):
         writer = PDGMWriter(f, "alpha", self.dim)
-        pairs, boundary_map_byte_sequence = self.compute_pairs(algorithm)
-        writer.save_pairs(pairs, self.index_to_level, output_suppl_info)
+
+        matrix = self.build_phat_matrix()
+        matrix.reduce(algorithm)
+
+        writer.save_pairs(matrix.birth_death_pairs(), self.index_to_level, output_suppl_info)
         writer.append_chunk(self.alpha_information_chunk())
 
         if output_suppl_info:
@@ -232,14 +266,10 @@ class AlphaFiltration(FiltrationWithIndex):
             writer.append_simple_chunk("index_to_simplex", self.index_to_simplex)
 
         if self.save_boundary_map:
-            writer.append_chunk(BinaryChunk("boundary_map",
-                                            boundary_map_byte_sequence))
+            writer.append_chunk(BinaryChunk("boundary_map", matrix.boundary_map_byte_sequence()))
         if self.save_phtrees:
-            boundary_map = msgpack.unpackb(
-                boundary_map_byte_sequence, raw=False
-            ).get("map")
-            writer.append_simple_chunk("phtrees",
-                                       self.build_phtrees(boundary_map))
+            boundary_map = msgpack.unpackb(matrix.boundary_map_byte_sequence(), raw=False).get("map")
+            writer.append_simple_chunk("phtrees", self.build_phtrees(boundary_map))
         writer.write()
 
     @cached_property
@@ -254,18 +284,10 @@ class AlphaFiltration(FiltrationWithIndex):
     def points_weighted(self):
         return self.point_weights is not None
 
-    @property
-    def index_map(self):
-        return IndexMapForAlphaFiltration(
-            np.array(self.index_to_level), self.points,
-            self.index_to_simplex, self.symbols, self.dim
-        )
-
     def build_phat_matrix(self):
         matrix = phat.Matrix(len(self.simplices), self.boundary_map_style())
         for simplex in self.simplices:
-            boundary = [self.dict_simplices[key].index
-                        for key in simplex.boundary_keys()]
+            boundary = [self.dict_simplices[key].index for key in simplex.boundary_keys()]
             matrix.set_dim_col(simplex.index, simplex.dim, boundary)
 
         return matrix
@@ -284,46 +306,3 @@ class AlphaFiltration(FiltrationWithIndex):
     @staticmethod
     def count_essential_pairs(pairs):
         return sum([1 for pair in pairs if pair[2] is None])
-
-    @staticmethod
-    def favorite_algorithm():
-        return "phat-twist"
-
-    def write_dipha_complex(self, output, use_index=True):
-        """"Write dipha bondary matrix data to file"""
-        def write_dipha_header(num_simplices):
-            """Write magic number, type, boundary/coboundary,
-            num_simplices, and dimension
-            """
-            output.write(struct.pack("qqqqq", 8067171840, 0, 0, num_simplices, self.dim))
-
-        def write_dimensions():
-            """Write dimensions of each simplex"""
-            for simplex in self.simplices:
-                output.write(struct.pack("q", simplex.dim))
-
-        def write_birth_radii():
-            """Write birth radii for each simplex"""
-            for simplex in self.simplices:
-                if use_index:
-                    output.write(struct.pack("d", simplex.index))
-                else:
-                    output.write(struct.pack("d", simplex.birth_radius))
-
-        def write_boundary_map_sizes():
-            n = 0
-            for simplex in self.simplices:
-                output.write(struct.pack("q", n))
-                n += len(simplex.boundary_keys())
-            output.write(struct.pack("q", n))  # Write the number of all boundary elements
-
-        def write_boundary_map():
-            for simplex in self.simplices:
-                for key in simplex.boundary_keys():
-                    output.write(struct.pack("q", self.dict_simplices[key].index))
-
-        write_dipha_header(len(self.simplices))
-        write_dimensions()
-        write_birth_radii()
-        write_boundary_map_sizes()
-        write_boundary_map()
