@@ -28,7 +28,7 @@ def setup_matrix(matrix, boundary_map):
     for i in range(len(map)):
         matrix.add_cell(dim(i))
         signed_column = [(j, sign(i, kth)) for kth, j in enumerate(column(i))]
-        for (j, s) in sorted(signed_column):
+        for j, s in sorted(signed_column):
             matrix.add_boundary_coef(i, j, s)
 
 
@@ -50,9 +50,7 @@ def main(args=None):
         return 0
     else:
         stop_time = diagram.index_to_level[stop_index]
-        print("Problematic coef: {}, Problematic time: {}".format(
-            stop_coef, stop_time)
-        )
+        print("Problematic coef: {}, Problematic time: {}".format(stop_coef, stop_time))
         return 1
 
 
