@@ -1,15 +1,15 @@
-from forwardable import forwardable
 import matplotlib.pyplot as plt
+import numpy as np
 
 import homcloud.plot_PD as plot_PD
 import homcloud.histogram as homhisto
-import homcloud.vectorize_PD as vectorize_PD
 import homcloud.plot_PD_slice as plot_PD_slice
 import homcloud.pdgm as pdgm
+from homcloud.delegate import forwardable
 
 
-@forwardable()
-class HistoSpec(object):
+@forwardable
+class HistoSpec:
     """
     This class represents a 2D mesh on the plane of birth-death pairs.
 
@@ -40,23 +40,23 @@ class HistoSpec(object):
 
     """
 
-    def __init__(self, x_range, xbins, y_range=None, ybins=None, pd=None,
-                 superlevel=False):
-        rulers = homhisto.Ruler.create_xy_rulers(x_range, xbins,
-                                                 y_range, ybins, pd)
+    def __init__(self, x_range, xbins, y_range=None, ybins=None, pd=None, superlevel=False):
+        rulers = homhisto.Ruler.create_xy_rulers(x_range, xbins, y_range, ybins, pd)
         self.spec = homhisto.HistoSpec.from_rulers(rulers[0], rulers[1], superlevel)
         # self.rulers = rulers
         # self.sign_flipped = superlevel
         self._histoinfo = None
 
-    def_delegators("spec", "sign_flipped")
+    __delegator_definitions__ = {"spec": "sign_flipped"}
 
     @classmethod
     def from_histoinfo(cls, histoinfo):
-        new_mesh = cls((histoinfo["x-edges"][0], histoinfo["x-edges"][-1]),
-                       len(histoinfo["x-edges"]) - 1,
-                       (histoinfo["y-edges"][0], histoinfo["y-edges"][-1]),
-                       len(histoinfo["y-edges"]) - 1,)
+        new_mesh = cls(
+            (histoinfo["x-edges"][0], histoinfo["x-edges"][-1]),
+            len(histoinfo["x-edges"]) - 1,
+            (histoinfo["y-edges"][0], histoinfo["y-edges"][-1]),
+            len(histoinfo["y-edges"]) - 1,
+        )
         new_mesh._histoinfo = histoinfo
         return new_mesh
 
@@ -85,6 +85,19 @@ class HistoSpec(object):
         """
         return Histogram(self.spec.histogram_from_vector(vector))
 
+    def histogram_from_2darray(self, values, ess_values=None):
+        """
+        Construct a histogram from a 2darray with additional ess_values 1darray.
+
+        Args:
+            values (numpy.ndarray): 2d histogram values.
+            ess_values (numpy.ndarray): 1d histogram values of essential births.
+
+        Returns:
+            :class:`Histogram`: The histogram.
+        """
+        return Histogram(self.spec.histogram_from_2darray(np.array(values), ess_values))
+
     def mask_from_vector(self, vector):
         """
         Construct a mask histogram from a boolean vector.
@@ -98,9 +111,7 @@ class HistoSpec(object):
             :class:`MaskHistogram`: The "histogram" whose values in bins are
                 True or False.
         """
-        return MaskHistogram(
-            self.spec.histogram_from_vector(vector, homhisto.BinaryHistogram)
-        )
+        return MaskHistogram(self.spec.histogram_from_vector(vector, homhisto.BinaryHistogram))
 
     def mask_from_2darray(self, array):
         return MaskHistogram(homhisto.BinaryHistogram(array, self.spec))
@@ -133,10 +144,15 @@ class PIVectorizeSpec(HistoSpec):
             of the bins on y-axis. Same as `x_range` if None is given.
         ybins (int or None): The number of bins on y-axis.
             Same as `x_bins` if None is given.
-        weight (tuple or string): The information of the weight function.
+        weight (tuple or string or Callable[[float, float], float]):
+            The information of the weight function.
+
             You can use one of the followings:
 
             * "none": A constant weight function
+            * Callable[[float, float], float]: Any callable object that computes weights.
+              The first and second arguments are birth and death times of a pair
+              and the return value is a positive weight value.
             * ("atan", `c`, `p`): An arctangent weight function,
               `c` and `p` should be floats. The weight function
               is atan(c * (death - birth)**p) .
@@ -151,8 +167,7 @@ class PIVectorizeSpec(HistoSpec):
             the parameter should be False.
     """
 
-    def __init__(self, x_range, xbins, y_range=None, ybins=None,
-                 weight=None, sigma=None, superlevel=False):
+    def __init__(self, x_range, xbins, y_range=None, ybins=None, weight=None, sigma=None, superlevel=False):
         super().__init__(x_range, xbins, y_range, ybins, None, superlevel)
         self.weight = self.weight_function(weight)
         self.sigma = sigma
@@ -163,11 +178,12 @@ class PIVectorizeSpec(HistoSpec):
             return lambda b, d: 1.0
         if isinstance(params, tuple):
             if params[0] == "atan":
-                return vectorize_PD.atan_weight_function(params[1], params[2])
+                return homhisto.atan_weight_function(params[1], params[2])
             if params[0] == "linear":
-                return vectorize_PD.linear_weight_function(1.0 / params[1])
-            raise(ValueError("Unknown weight type: {}".format(params[0])))
-        return params
+                return homhisto.linear_weight_function(1.0 / params[1])
+            raise ValueError("Unknown weight type: {}".format(params[0]))
+        if callable(params):
+            return params
 
     def vectorize(self, pd):
         """Vectroize `pd`.
@@ -200,25 +216,29 @@ class PIVectorizeSpec(HistoSpec):
 PIVectorizerMesh = PIVectorizeSpec
 
 
-@forwardable()
-class Histogram(object):
+@forwardable
+class Histogram:
     """The class represents a histogram on birth-death plane.
 
     Methods:
         x_range()
             Returns:
-                (tuple[int, int]): The lower and upper range of x-axis.
+                tuple[int, int]: The lower and upper range of x-axis.
 
         x_bins()
             Returns:
-                (int): The number of bins on x-axis.
+                int: The number of bins on x-axis.
 
         y_range()
             Returns:
-                (tuple[int, int]): The lower and upper range of y-axis.
+                tuple[int, int]: The lower and upper range of y-axis.
         y_bins()
             Returns:
-                (int): The number of bins on y-axis.
+                int: The number of bins on y-axis.
+        has_ess_values()
+            Returns:
+                bool: True if the histogram has information about
+                essential pairs, otherwise False
 
     Attributes:
         xedges (numpy.ndarray[x_bins + 1]): The edges of bins on x-axis
@@ -243,11 +263,34 @@ class Histogram(object):
     def values(self, values):
         self.orig.values = values
 
-    def_delegators('orig', 'x_range,y_range,x_bins,y_bins,xedges,yedges')
+    @property
+    def ess_values(self):
+        """
+        (numpy.ndarary, shape (x_bins,)): 1-dimensional array of values
+        in the bins of essential pairs.
+        """
+        return self.orig.ess_values
 
-    def plot(self, colorbar={}, style="colorhistogram", title="",
-             unit_name=None, font_size=None, aspect="equal", ax=None,
-             levels=None, plot_ess=False):
+    @ess_values.setter
+    def ess_values(self, ess_values):
+        self.orig.ess_values = ess_values
+
+    __delegator_definitions__ = {
+        "orig": ["x_range", "y_range", "x_bins", "y_bins", "xedges", "yedges", "has_ess_values"]
+    }
+
+    def plot(
+        self,
+        colorbar={},
+        style="colorhistogram",
+        title="",
+        unit_name=None,
+        font_size=None,
+        aspect="equal",
+        ax=None,
+        levels=None,
+        plot_ess=False,
+    ):
         """Plot a histogram by matplotlib.
 
         Args:
@@ -273,6 +316,7 @@ class Histogram(object):
                   specified, the value is determined by the maximum of
                   histogram values.
                 * "colormap" - matplotlib's colormap name
+                * "label" - The label of the colorbar.
 
             style (string): The plotting style. "colorhistogram" or "contour"
                 is available.
@@ -298,10 +342,11 @@ class Histogram(object):
             >>> plt.show()  # To show the histogram on your display
             >>> plt.savefig("histogram.png")  # To save the histogram figure.
         """
+        if not self.has_ess_values():
+            plot_ess = False
+
         plotter = plot_PD.PDPlotter.find_plotter(style)(
-            self.orig, self._zspec(colorbar), plot_PD.AuxPlotInfo(
-                title, unit_name, font_size, aspect, plot_ess
-            )
+            self.orig, self._zspec(colorbar), plot_PD.AuxPlotInfo(title, unit_name, font_size, aspect, plot_ess)
         )
         if levels is not None:
             plotter.levels = levels
@@ -313,23 +358,23 @@ class Histogram(object):
         ztype = colorbar.get("type", "linear")
         vmax = colorbar.get("max")
         vmin = colorbar.get("min")
+        label = colorbar.get("label")
         if "colormap" in colorbar:
             colormap = plt.get_cmap(colorbar["colormap"])
         else:
             colormap = None
         if ztype == "linear":
-            return plot_PD.ZSpec.Linear(vmax, vmin, colormap)
+            return plot_PD.ZSpec.Linear(vmax, vmin, colormap, label)
         elif ztype == "log":
-            return plot_PD.ZSpec.Log(vmax, vmin, colormap)
+            return plot_PD.ZSpec.Log(vmax, vmin, colormap, label)
         elif ztype == "loglog":
-            return plot_PD.ZSpec.LogLog(vmax, vmin, colormap)
+            return plot_PD.ZSpec.LogLog(vmax, vmin, colormap, label)
         elif ztype == "linear-midpoint":
-            return plot_PD.ZSpec.LinearMidPoint(colorbar["midpoint"],
-                                                vmax, vmin, colormap)
+            return plot_PD.ZSpec.LinearMidPoint(colorbar["midpoint"], vmax, vmin, colormap, label)
         elif ztype == "power":
-            return plot_PD.ZSpec.Power(colorbar["p"], vmax, vmin, colormap)
+            return plot_PD.ZSpec.Power(colorbar["p"], vmax, vmin, colormap, label)
         else:
-            raise(RuntimeError("unknown colorbar type: {}").format(ztype))
+            raise RuntimeError("unknown colorbar type: {}").format(ztype)
 
     @staticmethod
     def _fig_ax(ax):
@@ -363,8 +408,7 @@ class MaskHistogram(Histogram):
             In fact, this method can filter the list of :class:`OptimalVolume`
             by their birth and death times.
         """
-        return [pair for pair in pairs
-                if self.orig.value_at(pair.birth_time(), pair.death_time())]
+        return [pair for pair in pairs if self.orig.value_at(pair.birth_time(), pair.death_time())]
 
     def filter_pd(self, pd):
         """
@@ -382,7 +426,7 @@ class MaskHistogram(Histogram):
         return self.filter_pairs(pd.pairs())
 
 
-class SliceHistogram(object):
+class SliceHistogram:
     """
     This class represents a 1D histogram of birth-death pairs
     in the thin strip on a persistence diagram.
@@ -424,7 +468,9 @@ class SliceHistogram(object):
         ax.bar(self.edges[:-1], self.values, width=width, align="edge", log=logscale)
         ax.set_xlim(-width / 2, 1 + width / 2)
         ax.set_xticks([0, 1])
-        ax.set_xticklabels([
-            plot_PD_slice.construct_label(left_label, self.x1, self.y1),
-            plot_PD_slice.construct_label(right_label, self.x2, self.y2),
-        ])
+        ax.set_xticklabels(
+            [
+                plot_PD_slice.construct_label(left_label, self.x1, self.y1),
+                plot_PD_slice.construct_label(right_label, self.x2, self.y2),
+            ]
+        )
