@@ -3,7 +3,7 @@ import numpy as np
 import homcloud.maximal_0_component as maximal_0_component
 
 
-class PH0Components(object):
+class PH0Components:
     def __init__(self, pair, epsilon):
         self.pair = pair
         self.graph = maximal_0_component.Graph(pair.diagram.pd, pair.birth_index, pair.death_index, epsilon)
