@@ -4,7 +4,7 @@ from cached_property import cached_property
 import numpy as np
 
 
-class Graph(object):
+class Graph:
     def __init__(self, pdgm, birth_index, death_index, epsilon):
         self.pdgm = pdgm
         self.boundary_map_chunk = pdgm.boundary_map_chunk
