@@ -1,5 +1,4 @@
 import collections
-import msgpack
 import homcloud.optimal_one_cycle_ext as optimal_one_cycle_ext
 
 
@@ -8,15 +7,12 @@ def vertices(boundary_map, birth):
 
 
 def edges(boundary_map, birth):
-    return [
-        (index, boundary) for (index, (dim, boundary))
-        in enumerate(boundary_map["map"][:birth]) if dim == 1
-    ]
+    return [(index, boundary) for (index, (dim, boundary)) in enumerate(boundary_map["map"][:birth]) if dim == 1]
 
 
 def adjacent_vertices(boundary_map, birth):
     adjacent_vertices = {vertex: [] for vertex in vertices(boundary_map, birth)}
-    for (e, (x, y)) in edges(boundary_map, birth):
+    for e, (x, y) in edges(boundary_map, birth):
         adjacent_vertices[x].append((e, x, y))
         adjacent_vertices[y].append((e, y, x))
 
