@@ -13,7 +13,7 @@ def empty_pd(degree=0, sign_flipped=False):
     return SimplePDGM(degree, empty_array, empty_array, empty_array, sign_flipped)
 
 
-class PDGMBase(object):
+class PDGMBase:
     def minmax_of_birthdeath_time(self):
         return (min(self.births.min(), self.deaths.min()), max(self.births.max(), self.deaths.max()))
 
@@ -29,6 +29,10 @@ class PDGMBase(object):
             & (self.deaths <= death_max)
         )
 
+    @property
+    def num_pairs(self):
+        return len(self.births)
+
 
 class SimplePDGM(PDGMBase):
     def __init__(self, degree, births, deaths, ess_births=np.array([]), sign_flipped=False):
@@ -122,8 +126,8 @@ class PDGM(PDGMBase):
         return self.pdgmreader.metadata["dim"]
 
     @property
-    def num_pairs(self):
-        return len(self.births)
+    def pdgm_id(self):
+        return self.pdgmreader.pdgm_id
 
     @property
     def death_index_to_pair_number(self):
@@ -193,6 +197,16 @@ class PDGM(PDGMBase):
     def alpha_symbol_resolver(self):
         return self.simplicial_symbol_resolver
 
+    @cached_property
+    def vertex_index_resolver(self):
+        assert self.filtration_type in ["alpha", "simplicial"]
+
+        return geom_resolver.SimplicialResolver(
+            self.pdgmreader.load_simple_chunk("index_to_simplex"),
+            list(range(self.num_vertices)),
+            self.boundary_map,
+        )
+
     @cached_property
     def alpha_coord_resolver(self):
         assert self.filtration_type == "alpha"
@@ -203,6 +217,18 @@ class PDGM(PDGMBase):
             self.boundary_map,
         )
 
+    def periodic_alpha_coord_resolver(self, pbc_ratio):
+        assert self.filtration_type == "alpha"
+        assert self.alpha_information["periodicity"]
+
+        return geom_resolver.PeriodicSimplicialResolver(
+            self.pdgmreader.load_simple_chunk("index_to_simplex"),
+            self.pdgmreader.load_simple_chunk("vertex_coordintes"),
+            self.boundary_map,
+            self.alpha_information["periodicity"],
+            pbc_ratio,
+        )
+
     @cached_property
     def cubical_geometry_resolver(self):
         assert self.filtration_type == "cubical"
@@ -233,24 +259,61 @@ class PDGM(PDGMBase):
             self.pdgmreader.load_simple_chunk("vertex_symbols") or self.default_vertex_symbols()
         )
 
-    def geometry_resolver(self, use_symbol):
-        if self.filtration_type == "alpha":
-            if use_symbol:
+    @cached_property
+    def coupled_alpha_relative_coord_resolver(self):
+        assert self.filtration_type == "coupled-alpha-relative"
+
+        return geom_resolver.CoupledAlphaRelativeResolver(
+            self.pdgmreader.load_simple_chunk("index_to_simplex"),
+            self.pdgmreader.load_simple_chunk("vertex_coordintes"),
+        )
+
+    @cached_property
+    def coupled_alpha_relative_symbol_resolver(self):
+        assert self.filtration_type == "coupled-alpha-relative"
+
+        return geom_resolver.CoupledAlphaRelativeResolver(
+            self.pdgmreader.load_simple_chunk("index_to_simplex"),
+            self.pdgmreader.load_simple_chunk("vertex_symbols") or self.default_vertex_symbols(),
+        )
+
+    @cached_property
+    def coupled_alpha_relative_vertex_index_resolver(self):
+        assert self.filtration_type == "coupled-alpha-relative"
+
+        return geom_resolver.CoupledAlphaRelativeResolver(
+            self.pdgmreader.load_simple_chunk("index_to_simplex"),
+            list(range(self.num_vertices)),
+        )
+
+    def get_geometry_resolver(self, resolver_type="default", pbc_ratio=None):
+        match (self.filtration_type, resolver_type):
+            case ("alpha", "symbols"):
                 return self.alpha_symbol_resolver
-            else:
+            case ("alpha", "default" | "coordinates") if pbc_ratio is not None:
+                return self.periodic_alpha_coord_resolver(pbc_ratio)
+            case ("alpha", "default" | "coordinates"):
                 return self.alpha_coord_resolver
-        elif self.filtration_type == "simplicial":
-            return self.simplicial_symbol_resolver
-        elif self.filtration_type == "cubical":
-            return self.cubical_geometry_resolver
-        elif self.filtration_type == "bitmap":
-            return self.bitmap_geometry_resolver
-        elif self.filtration_type == "abstract":
-            return self.abstract_geometry_resolver
-        elif self.filtration_type == "rips":
-            return self.rips_geometry_resolver
-        else:
-            raise RuntimeError("Geometry resolver is unavailable for {}".format(self.filtration_type))
+            case ("alpha" | "simplicial", "vertex_indexes" | "vindexes"):
+                return self.vertex_index_resolver
+            case ("simplicial", "default"):
+                return self.simplicial_symbol_resolver
+            case ("cubical", "default"):
+                return self.cubical_geometry_resolver
+            case ("bitmap", "default"):
+                return self.bitmap_geometry_resolver
+            case ("abstract", "default"):
+                return self.abstract_geometry_resolver
+            case ("rips", "default" | "symbols"):
+                return self.rips_geometry_resolver
+            case ("coupled-alpha-relative", "default" | "coordinates"):
+                return self.coupled_alpha_relative_coord_resolver
+            case ("coupled-alpha-relative", "symbols"):
+                return self.coupled_alpha_relative_symbol_resolver
+            case ("coupled-alpha-relative", "vertex_indexes" | "vindexes"):
+                return self.coupled_alpha_relative_vertex_index_resolver
+            case _:
+                raise RuntimeError(f"Geometry resolver is unavailable for {self.filtration_type}, {resolver_type}")
 
     def boundary_map(self, cell_index):
         return self.boundary_map_chunk["map"][cell_index][1]
@@ -261,15 +324,15 @@ class PDGM(PDGMBase):
 
     @cached_property
     def birth_positions(self):
-        return self.geometry_resolver(False).resolve_cells(self.birth_indices)
+        return self.get_geometry_resolver().resolve_cells(self.birth_indices)
 
     @cached_property
     def death_positions(self):
-        return self.geometry_resolver(False).resolve_cells(self.death_indices)
+        return self.get_geometry_resolver().resolve_cells(self.death_indices)
 
     @cached_property
     def essential_birth_positions(self):
-        return self.geometry_resolver(False).resolve_cells(self.essential_birth_indices)
+        return self.get_geometry_resolver().resolve_cells(self.essential_birth_indices)
 
     def pairs_positions(self):
         return zip(self.births, self.deaths, self.birth_positions, self.death_positions)
