@@ -2,6 +2,7 @@ import msgpack
 import math
 import struct
 from pathlib import Path
+import uuid
 
 from cached_property import cached_property
 import numpy as np
@@ -9,10 +10,10 @@ import numpy as np
 MAGIC_HEADER = (b"HomCloud .pdgm format" + b"\0" * 24)[0:24]
 MAGIC_HEADER_LENGTH = len(MAGIC_HEADER)
 HEADER_LENGTH = MAGIC_HEADER_LENGTH + 8
-FORMAT_VERSION = 1
+FORMAT_VERSION = 2
 
 
-class PDChunk(object):
+class PDChunk:
     def __init__(self, chunktype, degree, births, deaths, ess_births):
         self.chunktype = chunktype
         self.degree = degree
@@ -22,66 +23,57 @@ class PDChunk(object):
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": self.chunktype, "degree": self.degree,
-            "births": self.births, "deaths": self.deaths,
-            "essential_births": self.ess_births
-        }, use_bin_type=True)
+        return msgpack.packb(
+            {
+                "chunktype": self.chunktype,
+                "degree": self.degree,
+                "births": self.births,
+                "deaths": self.deaths,
+                "essential_births": self.ess_births,
+            },
+            use_bin_type=True,
+        )
 
     def metadata(self, start):
-        return {
-            "chunktype": self.chunktype, "degree": self.degree, "start": start,
-            "length": len(self.binary)
-        }
+        return {"chunktype": self.chunktype, "degree": self.degree, "start": start, "length": len(self.binary)}
 
 
-class SimpleChunk(object):
+class SimpleChunk:
     def __init__(self, chunktype, data):
         self.chunktype = chunktype
         self.data = data
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": self.chunktype, "data": self.data
-        }, use_bin_type=True)
+        return msgpack.packb({"chunktype": self.chunktype, "data": self.data}, use_bin_type=True)
 
     def metadata(self, start):
-        return {
-            "chunktype": self.chunktype, "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": self.chunktype, "start": start, "length": len(self.binary)}
 
 
-class BinaryChunk(object):
+class BinaryChunk:
     def __init__(self, chunktype, binary):
         self.chunktype = chunktype
         self.binary = binary
 
     def metadata(self, start):
-        return {
-            "chunktype": self.chunktype, "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": self.chunktype, "start": start, "length": len(self.binary)}
 
 
-class AllPairsChunk(object):
+class AllPairsChunk:
     def __init__(self, degree, pairs):
         self.degree = degree
         self.pairs = pairs
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "allpairs", "degree": self.degree, "data": self.pairs
-        })
+        return msgpack.packb({"chunktype": "allpairs", "degree": self.degree, "data": self.pairs})
 
     def metadata(self, start):
-        return {
-            "chunktype": "allpairs", "degree": self.degree, "start": start,
-            "length": len(self.binary)
-        }
+        return {"chunktype": "allpairs", "degree": self.degree, "start": start, "length": len(self.binary)}
 
 
-class SimpleDegreeChunk(object):
+class SimpleDegreeChunk:
     def __init__(self, chunktype, degree, data):
         self.chunktype = chunktype
         self.degree = degree
@@ -89,98 +81,114 @@ class SimpleDegreeChunk(object):
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": self.chunktype, "degree": self.degree, "data": self.data
-        })
+        return msgpack.packb({"chunktype": self.chunktype, "degree": self.degree, "data": self.data})
 
     def metadata(self, start):
-        return {
-            "chunktype": self.chunktype, "degree": self.degree, "start": start,
-            "length": len(self.binary)
-        }
+        return {"chunktype": self.chunktype, "degree": self.degree, "start": start, "length": len(self.binary)}
 
 
-class BitmapInformationChunk(object):
+class BitmapInformationChunk:
     def __init__(self, shape, periodicity):
         self.shape = shape
         self.periodicity = periodicity
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "bitmap_information",
-            "shape": self.shape, "periodicity": self.periodicity,
-        })
+        return msgpack.packb(
+            {
+                "chunktype": "bitmap_information",
+                "shape": self.shape,
+                "periodicity": self.periodicity,
+            }
+        )
 
     def metadata(self, start):
-        return {
-            "chunktype": "bitmap_information", "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": "bitmap_information", "start": start, "length": len(self.binary)}
 
 
-class AlphaInformationChunk(object):
-    def __init__(self, num_vertices, periodicity):
+class AlphaInformationChunk:
+    def __init__(self, num_vertices, periodicity, weighted, squared):
         self.num_vertices = num_vertices
         self.periodicity = periodicity
+        self.weighted = weighted
+        self.squared = squared
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "alpha_information",
-            "num_vertices": self.num_vertices, "periodicity": self.periodicity,
-        })
+        return msgpack.packb(
+            {
+                "chunktype": "alpha_information",
+                "num_vertices": self.num_vertices,
+                "periodicity": self.periodicity,
+                "weighted": self.weighted,
+                "squared": self.squared,
+            }
+        )
 
     def metadata(self, start):
-        return {
-            "chunktype": "alpha_information", "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": "alpha_information", "start": start, "length": len(self.binary)}
 
 
-class RipsInformationChunk(object):
+class RipsInformationChunk:
     def __init__(self, num_vertices):
         self.num_vertices = num_vertices
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "rips_information",
-            "num_vertices": self.num_vertices
-        })
+        return msgpack.packb({"chunktype": "rips_information", "num_vertices": self.num_vertices})
 
     def metadata(self, start):
-        return {
-            "chunktype": "rips_information", "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": "rips_information", "start": start, "length": len(self.binary)}
 
 
-class BoundaryMapChunk(object):
+class CoupledAlphaRelativeInformationChunk:
+    def __init__(self, num_vertices_X, num_vertices_Y, squared):
+        self.num_vertices_X = num_vertices_X
+        self.num_vertices_Y = num_vertices_Y
+        self.squared = squared
+
+    @cached_property
+    def binary(self):
+        return msgpack.packb(
+            {
+                "chunktype": "coupled_alpha_relative_information",
+                "num_vertices_X": self.num_vertices_X,
+                "num_vertices_Y": self.num_vertices_Y,
+                "squared": self.squared,
+            }
+        )
+
+    def metadata(self, start):
+        return {"chunktype": "coupled_alpha_relative_information", "start": start, "length": len(self.binary)}
+
+
+class BoundaryMapChunk:
     def __init__(self, type, map):
         self.type = type
         self.map = map
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "boundary_map", "type": self.type, "map": self.map
-        })
+        return msgpack.packb({"chunktype": "boundary_map", "type": self.type, "map": self.map})
 
     def metadata(self, start):
-        return {
-            "chunktype": "boundary_map", "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": "boundary_map", "start": start, "length": len(self.binary)}
 
 
-class GraphWeightsChunk(object):
+class GraphWeightsChunk:
     def __init__(self, matrix):
         assert matrix.ndim == 2 and matrix.shape[0] == matrix.shape[1]
         self.matrix = matrix
 
     @cached_property
     def binary(self):
-        return msgpack.packb({
-            "chunktype": "graph_weights", "num_vertices": self.num_vertices,
-            "weights": self.graph_weights(),
-        })
+        return msgpack.packb(
+            {
+                "chunktype": "graph_weights",
+                "num_vertices": self.num_vertices,
+                "weights": self.graph_weights(),
+            }
+        )
 
     def graph_weights(self):
         return self.matrix[np.triu_indices(self.num_vertices, 1)].tolist()
@@ -190,18 +198,21 @@ class GraphWeightsChunk(object):
         return self.matrix.shape[0]
 
     def metadata(self, start):
-        return {
-            "chunktype": "graph_weights", "start": start, "length": len(self.binary)
-        }
+        return {"chunktype": "graph_weights", "start": start, "length": len(self.binary)}
 
 
-class PDGMWriter(object):
-    def __init__(self, out, filtration_type, dim=None):
+class PDGMWriter:
+    def __init__(self, out, filtration_type, dim=None, pdgm_id=None):
         self.out = out
         self.filtration_type = filtration_type
         self.sign_flipped = False
         self.dim = dim
         self.chunks = []
+        self.pdgm_id = pdgm_id or self.generate_pdgm_id()
+
+    @staticmethod
+    def generate_pdgm_id():
+        return uuid.uuid4().hex
 
     def append_chunk(self, chunk):
         self.chunks.append(chunk)
@@ -238,7 +249,8 @@ class PDGMWriter(object):
             "filtration_type": self.filtration_type,
             "sign_flipped": self.sign_flipped,
             "dim": self.dim,
-            "chunks": self.chunk_metadata_list()
+            "pdgm_id": self.pdgm_id,
+            "chunks": self.chunk_metadata_list(),
         }
 
     def chunk_metadata_list(self):
@@ -253,23 +265,21 @@ class PDGMWriter(object):
 
     def save_pairs(self, pairs, index_to_levels, save_index_info=True):
         indexed_pds = [IndexedPD(d, index_to_levels) for d in range(self.dim + 1)]
-        for (d, birth, death) in pairs:
+        for d, birth, death in pairs:
             if d > self.dim:
                 continue
             indexed_pds[d].append(birth, death)
 
-        self.chunks.extend(indexed_pd.to_unindexed_chunk()
-                           for indexed_pd in indexed_pds)
+        self.chunks.extend(indexed_pd.to_unindexed_chunk() for indexed_pd in indexed_pds)
 
         if not save_index_info:
             return
 
         self.chunks.extend(indexed_pd.to_chunk() for indexed_pd in indexed_pds)
-        self.chunks.extend(indexed_pd.to_allpairs_chunk()
-                           for indexed_pd in indexed_pds)
+        self.chunks.extend(indexed_pd.to_allpairs_chunk() for indexed_pd in indexed_pds)
 
 
-class IndexedPD(object):
+class IndexedPD:
     def __init__(self, degree, index_to_levels):
         self.degree = degree
         self.birth_indices = []
@@ -303,8 +313,7 @@ class IndexedPD(object):
         return self.index_to_levels[index] if index is not None else None
 
     def to_chunk(self):
-        return PDChunk("indexed_pd", self.degree, self.birth_indices, self.death_indices,
-                       self.ess_birth_indices)
+        return PDChunk("indexed_pd", self.degree, self.birth_indices, self.death_indices, self.ess_birth_indices)
 
     def to_unindexed_chunk(self):
         return PDChunk("pd", self.degree, self.births, self.deaths, self.ess_births)
@@ -313,7 +322,7 @@ class IndexedPD(object):
         return AllPairsChunk(self.degree, self.pairs)
 
 
-class PDGMReader(object):
+class PDGMReader:
     def __init__(self, infile, path=None):
         self.infile = infile
         self.path = path
@@ -332,18 +341,21 @@ class PDGMReader(object):
 
     def scan_magic_header(self):
         if self.infile.read(MAGIC_HEADER_LENGTH) != MAGIC_HEADER:
-            raise(RuntimeError("HomCloud .pdgm format magic header not found"))
+            raise RuntimeError("HomCloud .pdgm format magic header not found")
 
     def load_metadata(self):
-        metadata_length, = struct.unpack("q", self.infile.read(8))
-        self.metadata = msgpack.unpackb(self.infile.read(metadata_length),
-                                        raw=False)
+        (metadata_length,) = struct.unpack("q", self.infile.read(8))
+        self.metadata = msgpack.unpackb(self.infile.read(metadata_length), raw=False)
         self.pos_first_chunk = HEADER_LENGTH + metadata_length
 
     @property
     def filtration_type(self):
         return self.metadata["filtration_type"]
 
+    @property
+    def pdgm_id(self):
+        return self.metadata.get("pdgm_id")
+
     def unpack_chunk(self, chunk_metadata):
         return msgpack.unpackb(self.read_chunk_bytes(chunk_metadata), raw=False)
 
@@ -382,8 +394,7 @@ class PDGMReader(object):
 
     def find_chunk_metadata(self, chunktype, degree=None):
         def cond(chunk):
-            return ((chunk["chunktype"] == chunktype) and
-                    ((degree is None) or (chunk["degree"] == degree)))
+            return (chunk["chunktype"] == chunktype) and ((degree is None) or (chunk["degree"] == degree))
 
         return next(filter(cond, self.metadata["chunks"]), None)
 
