@@ -1,8 +1,9 @@
 import numpy as np
 
 from scipy.ndimage import distance_transform_cdt, distance_transform_edt
-from sklearn.neighbors import BallTree, DistanceMetric
-from scipy.ndimage.morphology import binary_erosion
+from sklearn.neighbors import BallTree
+from sklearn.metrics import DistanceMetric
+from scipy.ndimage import binary_erosion
 
 from homcloud.distance_transform_ext import positive_distance_transform_manhattan_with_mask
 from homcloud.pict.utils import nd_indices
@@ -37,7 +38,7 @@ def distance_transform(pict, metric, signed=False, periodicity=None, obstacle=No
     elif metric == "mahalanobis":
         ret[npict] = distance_transform_mahalanobis(npict, **opts)[npict]
     else:
-        raise(ValueError("Unknown metric: {}".format(metric)))
+        raise ValueError("Unknown metric: {}".format(metric))
 
     return ret
 
@@ -63,22 +64,19 @@ def distance_transform_mahalanobis(npict, VI=None, V=None):
 
 def check_arguments_for_obstacle(metric, pict, obstacle):
     if metric != "manhattan":
-        raise(RuntimeError("obstacle is avaiable only with manhattan metric"))
+        raise RuntimeError("obstacle is avaiable only with manhattan metric")
     if np.any(pict & obstacle):
-        raise(RuntimeError("obstacle must not have intersection with input image"))
+        raise RuntimeError("obstacle must not have intersection with input image")
 
 
 def pixel_positions(pict):
-    """Return indices of True in 2D boolean ndarray pict
-    """
+    """Return indices of True in 2D boolean ndarray pict"""
     indices = nd_indices(pict.shape)
     return indices[pict.flatten()]
 
 
 def boundary_pixels(pict):
-    return pixel_positions(
-        np.logical_xor(pict, binary_erosion(pict, iterations=1, border_value=1))
-    )
+    return pixel_positions(np.logical_xor(pict, binary_erosion(pict, iterations=1, border_value=1)))
 
 
 def periodic_extend(pict, periodicity):
