@@ -2,10 +2,6 @@ import collections
 
 import numpy as np
 
-from homcloud.visualize_3d import ParaViewLinesDrawer
-from homcloud.index_map import MapType
-# import homcloud.diagram
-
 
 def search(bitmap, birth_level, start):
     ndim = bitmap.ndim
@@ -30,7 +26,7 @@ def search(bitmap, birth_level, start):
                 if next_pos[i] < 0 or next_pos[i] >= shape[i]:
                     continue
                 if bitmap[next_pos] < birth_level:
-                    yield(next_pos)
+                    yield next_pos
 
     def step_from(head_name, at):
         for next_pos in next_from(at):
@@ -46,7 +42,7 @@ def search(bitmap, birth_level, start):
                 queue.append((head_name, next_pos))
 
     def initialize():
-        for (head_name, pos) in enumerate(next_from(start)):
+        for head_name, pos in enumerate(next_from(start)):
             memo[pos] = (head_name, start)
             queue.append((head_name, pos))
 
@@ -61,21 +57,24 @@ def search(bitmap, birth_level, start):
             return result
 
 
-class Finder(object):
-    def __init__(self, diagram):
-        self.diagram = diagram
-        self.index_map = diagram.index_map
-        self.bitmap = diagram.index_map.indexed_bitmap()
-        assert self.index_map.type() == MapType.bitmap
+class Finder:
+    def __init__(self, pdgm):
+        assert pdgm.filtration_type == "bitmap"
+        self.pdgm = pdgm
 
     def query_pair(self, birth, death):
-        path = search(self.bitmap, birth, tuple(self.index_map.positions[birth]))
-        return Result(self.index_map.resolve_level(birth),
-                      self.index_map.resolve_level(death),
-                      path)
+        path = search(self.bitmap, birth, self.start_pixel(birth))
+        return Result(self.pdgm.index_to_level[birth], self.pdgm.index_to_level[death], path)
+
+    @property
+    def bitmap(self):
+        return self.pdgm.indexed_bitmap
+
+    def start_pixel(self, birth):
+        return tuple(self.pdgm.index_to_pixel[birth])
 
 
-class Result(object):
+class Result:
     def __init__(self, birth_time, death_time, path):
         self.birth_time = birth_time
         self.death_time = death_time
@@ -86,15 +85,8 @@ class Result(object):
             "birth-time": self.birth_time,
             "death-time": self.death_time,
             "path": list(map(list, self.path)),
-            "boundary-points": list(map(list, self.boundary_points()))
+            "boundary-points": list(map(list, self.boundary_points())),
         }
 
     def boundary_points(self):
         return set(self.path)
-
-    def draw(self, drawer, color, index):
-        drawer.draw_loop(self.path, color, index=index)
-
-
-def prepare_drawer_for_paraview(n_colors):
-    return ParaViewLinesDrawer(n_colors, {"index": None, "isboundary": "1"})
