@@ -1,6 +1,7 @@
 """
 This module provides helper functions for pyvista (https://pyvista.org/) for HomCloud.
 """
+
 import pyvista as pv
 import numpy as np
 
@@ -13,12 +14,12 @@ def Bitmap3D(array):
         array (numpy.ndarray[(Any, Any, Any), float]): 3d array
 
     Returns:
-        pyvista.UniformGrid: PyVista's mesh object
+        pyvista.ImageData: PyVista's mesh object
 
     """
     array = np.array(array)
     assert array.ndim == 3
-    grids = pv.UniformGrid(dimensions=np.array(array.shape) + 1, origin=(-0.5, -0.5, -0.5))
+    grids = pv.ImageData(dimensions=np.array(array.shape) + 1, origin=(-0.5, -0.5, -0.5))
     grids.cell_data["values"] = array.flatten(order="F")
     return grids
 
@@ -73,7 +74,6 @@ def Triangles(triangles):
     return pv.PolyData(
         triangles.reshape((triangles.shape[0] * 3, 3)),
         faces=[[3, k * 3, k * 3 + 1, k * 3 + 2] for k in range(triangles.shape[0])],
-        n_faces=triangles.shape[0],
     )
 
 
