@@ -3,7 +3,7 @@ import pytest
 import homcloud.phat_ext as phat
 
 
-class TestMatrix(object):
+class TestMatrix:
     @pytest.fixture
     def matrix(self):
         matrix = phat.Matrix(7, "simplicial")
